""" The pdelab specific parts of the code generation process """

# Trigger some imports that are needed to have all backend implementations visible
# to the selection mechanisms
from dune.perftool.generation import (get_backend)
from dune.perftool.options import option_switch
from dune.perftool.pdelab.argument import (pymbolic_apply_function,
                                           pymbolic_apply_function_gradient,
                                           pymbolic_trialfunction,
                                           pymbolic_trialfunction_gradient,
                                           pymbolic_coefficient
                                           )
from dune.perftool.pdelab.basis import (pymbolic_basis,
                                        pymbolic_reference_gradient,
                                        )
from dune.perftool.pdelab.geometry import (component_iname,
                                           name_cell_volume,
                                           name_facet_area,
                                           pymbolic_facet_jacobian_determinant,
                                           pymbolic_jacobian_determinant,
                                           pymbolic_jacobian_inverse_transposed,
                                           name_unit_inner_normal,
                                           name_unit_outer_normal,
                                           to_global,
                                           )
from dune.perftool.pdelab.index import (name_index,
                                        )
from dune.perftool.pdelab.parameter import (cell_parameter_function,
                                            intersection_parameter_function,
                                            )
from dune.perftool.pdelab.quadrature import (pymbolic_quadrature_weight,
                                             pymbolic_quadrature_position,
                                             quadrature_inames,
                                             )
from dune.perftool.pdelab.spaces import (lfs_inames,
                                         )
from dune.perftool.pdelab.tensors import pymbolic_list_tensor, pymbolic_identity


class PDELabInterface(object):
    def __init__(self):
        # The visitor instance will be registered by its init method
        self.visitor = None

    #
    # TODO: The following ones are actually entirely PDELab independent!
    # They should be placed elsewhere and be used directly in the visitor.
    #

    def component_iname(self, context=None, count=None):
        return component_iname(context=context, count=count)

    def name_index(self, ind):
        return name_index(ind)

    #
    # Local function space related generator functions
    #

    def lfs_inames(self, element, restriction, number=None, context=''):
        return lfs_inames(element, restriction, number, context)

    #
    # Test and trial function related generator functions
    #

    # TODO: should pymbolic_basis also pass the interface?
    def pymbolic_basis(self, element, restriction, number, context=''):
        return pymbolic_basis(element, restriction, number, context)

    # TODO: should pymbolic_reference_gradient also pass the interface?
    def pymbolic_reference_gradient(self, element, restriction, number, context=''):
        return pymbolic_reference_gradient(element, restriction, number, context)

    def pymbolic_trialfunction_gradient(self, element, restriction, tree_path):
        return pymbolic_trialfunction_gradient(self, element, restriction, tree_path)

    def pymbolic_apply_function_gradient(self, element, restriction, tree_path):
        return pymbolic_apply_function_gradient(self, element, restriction, tree_path)

    def pymbolic_trialfunction(self, element, restriction, tree_path):
        return pymbolic_trialfunction(self, element, restriction, tree_path)

    def pymbolic_apply_function(self, element, restriction, tree_path):
        return pymbolic_apply_function(self, element, restriction, tree_path)

    #
    # Parameter function related generator functions
    #

    def intersection_parameter_function(self, name, expr, cellwise_constant):
        return intersection_parameter_function(name, expr, cellwise_constant)

    def cell_parameter_function(self, name, expr, restriction, cellwise_constant):
        return cell_parameter_function(name, expr, restriction, cellwise_constant)

    #
    # Tensor expression related generator functions
    #

    def pymbolic_list_tensor(self, o):
        return pymbolic_list_tensor(o, self.visitor)

    def pymbolic_identity(self, o):
        return pymbolic_identity(o)

    #
    # Geometry related generator functions
    #

    def pymbolic_spatial_coordinate(self):
        return to_global(pymbolic_quadrature_position())

    def pymbolic_facet_jacobian_determinant(self):
        return pymbolic_facet_jacobian_determinant()

    def pymbolic_jacobian_determinant(self):
        return pymbolic_jacobian_determinant()

    def pymbolic_jacobian_inverse_transposed(self, i, j, restriction):
        return pymbolic_jacobian_inverse_transposed(i, j, restriction)

    def name_unit_inner_normal(self):
        return name_unit_inner_normal()

    def name_unit_outer_normal(self):
        return name_unit_outer_normal()

    def name_cell_volume(self, restriction):
        return name_cell_volume(restriction)

    def name_facet_area(self):
        return name_facet_area()

    #
    # Quadrature related generator functions
    #

    def pymbolic_quadrature_weight(self):
        return pymbolic_quadrature_weight()

    # TODO Should this be part of interface or not?
    def quadrature_inames(self):
        return quadrature_inames()
